#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <errno.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include "cmd.h"

int send_cmd(int s, unsigned char cmd, unsigned int data_sz, void *data);

int main(int argc, char **argv)
{
	char *addr_str = 0;
	int s, i, port = 7878;
	struct sockaddr_in addr;

	for(i=1; i<argc; i++) {
		if(argv[i][0] == '-' && argv[i][2] == 0) {
			switch(argv[i][1]) {
			case 'p':
				if(!isdigit(argv[++i][0])) {
					fprintf(stderr, "-p must be followed by a port number\n");
					return EXIT_FAILURE;
				}
				port = atoi(argv[i]);
				break;

			default:
				fprintf(stderr, "unrecognized argument: %s\n", argv[i]);
				return EXIT_FAILURE;
			}
		} else {
			if(addr_str) {
				fprintf(stderr, "you can only specify the address once\n");
				return EXIT_FAILURE;
			}
			addr_str = argv[i];
		}
	}

	if(!addr_str) {
		addr_str = "127.0.0.1";
	}


	if((s = socket(PF_INET, SOCK_STREAM, 0)) == -1) {
		perror("failed to create socket");
		return EXIT_FAILURE;
	}

	memset(&addr, 0, sizeof addr);
	addr.sin_family = AF_INET;
	addr.sin_port = htons(port);
	addr.sin_addr.s_addr = inet_addr(addr_str);

	/* the ip is invalid, try treating it as a host name */
	if(addr.sin_addr.s_addr == -1) {
		struct hostent *host = gethostbyname(addr_str);
		if(!host) {
			perror("name resolution failed");
			return EXIT_FAILURE;
		}
		addr.sin_addr = *((struct in_addr*)host->h_addr);
	}

	if(connect(s, (struct sockaddr*)&addr, sizeof addr) == -1) {
		perror("could not connect to server");
		return EXIT_FAILURE;
	}


	send_cmd(s, OPEN, 0, 0);
	send_cmd(s, CLOSE, 0, 0);
	send_cmd(s, READ, 0, 0);
	send_cmd(s, WRITE, 0, 0);

	close(s);
	return 0;
}

int send_cmd(int s, unsigned char cmd, unsigned int data_sz, void *data)
{
	if(data) {
		cmd |= BIT_ADD;
	}

	write(s, &cmd, 1);

	if(data) {
		write(s, &data_sz, sizeof data_sz);
		write(s, data, data_sz);
	}

	return 0;
}
