#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <signal.h>
#include <assert.h>
#include <unistd.h>
#include <wait.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include "cmd.h"


struct client {
	pid_t pid;
	struct client *next;
};

void handle_con(int s);
void sig_handler(int sig);
void child_sig_handler(int sig);

struct client *clist;
int csock;

int main(int argc, char **argv)
{
	int lis, port = 7878;
	struct sockaddr_in addr;

	/* client list dummy node */
	clist = malloc(sizeof *clist);
	clist->pid = 0;
	clist->next = 0;

	if((lis = socket(PF_INET, SOCK_STREAM, 0)) == -1) {
		perror("failed to create listening socket");
		return EXIT_FAILURE;
	}

	memset(&addr, 0, sizeof addr);
	addr.sin_family = AF_INET;
	addr.sin_port = htons(port);
	addr.sin_addr.s_addr = htonl(INADDR_ANY);

	if(bind(lis, (struct sockaddr*)&addr, sizeof addr) == -1) {
		fprintf(stderr, "failed to bind to port %d: %s\n", port, strerror(errno));
		return EXIT_FAILURE;
	}
	listen(lis, 32);

	signal(SIGCHLD, sig_handler);
	signal(SIGINT, sig_handler);

	for(;;) {
		int s;
		pid_t pid;

		printf("waiting for connections\n");
		if((s = accept(lis, 0, 0)) == -1) {
			perror("wait for connection failed");
			continue;
		}

		if((pid = fork())) {
			struct client *cnode;
			
			close(s);

			cnode = malloc(sizeof *cnode);
			cnode->pid = pid;
			cnode->next = clist->next;
			clist->next = cnode;
		} else {
			close(lis);
			signal(SIGINT, child_sig_handler);
			handle_con(s);
			printf("foooooo\n");
			exit(0);
		}	
	}
}

void handle_con(int s)
{
	csock = s;

	for(;;) {
		unsigned char cmd, *buf;
		unsigned int data_sz = 0;
		ssize_t rdsz;

		if((rdsz = read(s, &cmd, 1)) < 1) {
			if(rdsz == 0 || (rdsz == -1 && errno == EPIPE)) {
				close(s);
				return;
			} else if(rdsz == -1) {
				if(errno == EINTR) {
					continue;
				} else {
					perror("read failed");
					continue;
				}
			}
		}

		/* check for additional data */
		if(cmd & BIT_ADD) {
			ssize_t rd_data = 0;

			printf("adddddddddd\n");

			assert(read(s, &data_sz, sizeof data_sz) == sizeof data_sz);

			if(!(buf = malloc(data_sz))) {
				perror("malloc failed");
				close(s);
				exit(EXIT_FAILURE);
			}

			while(rd_data < data_sz) {
				if((rdsz = read(s, &buf, data_sz - rd_data)) < 0) {
					if(errno == EINTR) {
						continue;
					} else {
						perror("failed while reading additional data");
						break;
					}
				}
				rd_data += rdsz;
			}
		}

		/* print debug output */
		printf("cmd: %s (%d add data)\n", cmdstr[cmd], data_sz);
	}
}

void sig_handler(int sig)
{
	struct client *cnode;
	signal(sig, sig_handler);
		
	printf("sig_handler called: %d\n", sig);

	switch(sig) {
	case SIGCHLD:
		wait(0);
		break;

	case SIGINT:
		signal(SIGCHLD, SIG_IGN);
		cnode = clist->next;
		while(cnode) {
			kill(cnode->pid, SIGINT);
			wait(0);
			cnode = cnode->next;
		}
		exit(0);
		break;

	default:
		break;
	}
}

void child_sig_handler(int sig)
{
	printf("child sig handler: %d\n", sig);
	close(csock);
	exit(0);
}
